@@ -1,5 +1,5 @@
 /* rndw32.c  -	W32 entropy gatherer
- *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *	Copyright Peter Gutmann, Matt Thomlinson and Blake Coverett 1996-1999
  *
  * This file is part of GnuPG.
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
  *************************************************************************
  * The code here is based on code from Cryptlib 3.0 beta by Peter Gutmann.
@@ -70,244 +71,37 @@
 
 
 #include "types.h"
-#include "g10lib.h"
-#include "dynload.h"
+#include "util.h"
+#include "algorithms.h"
 
-/* We do not use the netropy DLL anymore because a standalone program is
- * easier to maintain and */
-/*#define USE_ENTROPY_DLL*/
-
-
-
-#ifdef IS_MODULE
-  #define _(a) (a)
-#else
-  #include "i18n.h"
-#endif
+#include "i18n.h"
 
 
 static int debug_me;
 
-#ifdef USE_ENTROPY_DLL
-
-#define WIN32_SLOW_SEEDER	0
-#define WIN32_FAST_SEEDER	1
-
-#define PCP_SUCCESS		0
-#define PCP_NULL_POINTER	1
-#define PCP_SEEDER_FAILED	2
-#define PCP_SEEDER_NO_MEM	3
-#define PCP_SEEDER_TOO_SMALL	4
-#define PCP_DLL_LOAD_FAILED	5
-#define PCP_UNKNOWN_PLATFORM	6
-#define PCP_ERROR_VERSION	7
-#define PCP_DLL_FUNC		8
-#define PCP_UNKNOWN_SEEDER_TYPE 9
-
-
-/****************
- * We sometimes get a SEEDER_TOO_SMALL error, in which case we increment
- * the internal buffer by SEEDER_INC_CHUNK until we reach MAX_SEEDER_SIZE
- * MAX_SEEDER_SIZE is used as an arbitrary limit to protect against
- * bugs in Winseed.
- */
-#define MAX_SEEDER_SIZE  500000
-#define SEEDER_INC_CHUNK  50000
-
-
-typedef void *WIN32_SEEDER;
-
-static WIN32_SEEDER (WINAPI *create_instance)( byte type, unsigned int *reason);
-static void	    (WINAPI *delete_instance)( WIN32_SEEDER that );
-static unsigned int (WINAPI *get_internal_seed_size)( WIN32_SEEDER that );
-static void	    (WINAPI *set_internal_seed_size)( WIN32_SEEDER that,
-						      unsigned int new_size);
-static unsigned int (WINAPI *get_expected_seed_size)( WIN32_SEEDER that);
-static unsigned int (WINAPI *get_seed)( WIN32_SEEDER that, byte *buffer,
-					unsigned int *desired_length);
-
-static WIN32_SEEDER slow_seeder, fast_seeder;
-static byte *entropy_buffer;
-static size_t entropy_buffer_size;
-
-/****************
- * Load and initialize the winseed DLL
- * NOTE: winseed is not part of the GnuPG distribution.  It should be available
- * at the GNU crypto FTP server site.
- * We do not load the DLL on demand to have a better control over the
- * location of the library.
- */
-static void
-load_and_init_winseed( void )
-{
-    HANDLE hInstance;
-    void *addr;
-    unsigned int reason = 0;
-    unsigned int n1, n2;
-    const char *dllname;
-
-    dllname = read_w32_registry_string( "HKEY_LOCAL_MACHINE",
-					"Software\\GNU\\GnuPG",
-					"EntropyDLL" );
-    if( !dllname )
-	dllname = "c:/gnupg/entropy.dll";
-
-    hInstance = LoadLibrary( dllname );
-    if( !hInstance )
-	goto failure;
-    if( !(addr = GetProcAddress( hInstance, "WS_create_instance" )) )
-	goto failure;
-    create_instance = addr;
-    if( !(addr = GetProcAddress( hInstance, "WS_delete_instance" )) )
-	goto failure;
-    delete_instance = addr;
-    if( !(addr = GetProcAddress( hInstance, "WS_get_internal_seed_size" )) )
-	goto failure;
-    get_internal_seed_size = addr;
-    if( !(addr = GetProcAddress( hInstance, "WS_set_internal_seed_size" )) )
-	goto failure;
-    set_internal_seed_size = addr;
-    if( !(addr = GetProcAddress( hInstance, "WS_get_expected_seed_size" )) )
-	goto failure;
-    get_expected_seed_size = addr;
-    if( !(addr = GetProcAddress( hInstance, "WS_get_seed" )) )
-	goto failure;
-    get_seed = addr;
-
-    /* we have all the functions - init the system */
-    slow_seeder = create_instance( WIN32_SLOW_SEEDER, &reason);
-    if( !slow_seeder ) {
-	g10_log_fatal("error creating winseed slow seeder: rc=%u\n", reason );
-	goto failure;
-    }
-    fast_seeder = create_instance( WIN32_FAST_SEEDER, &reason);
-    if( !fast_seeder ) {
-	g10_log_fatal("error creating winseed fast seeder: rc=%u\n", reason );
-	goto failure;
-    }
-    n1 = get_internal_seed_size( slow_seeder );
-    /*g10_log_info("slow buffer size=%u\n", n1);*/
-    n2 = get_internal_seed_size( fast_seeder );
-    /*g10_log_info("fast buffer size=%u\n", n2);*/
-
-    entropy_buffer_size =  n1 > n2? n1: n2;
-    entropy_buffer = m_alloc( entropy_buffer_size );
-    /*g10_log_info("using a buffer of size=%u\n", entropy_buffer_size );*/
-
-    return;
-
-  failure:
-    g10_log_fatal("error loading winseed DLL `%s'\n", dllname );
-}
-
-
-
-
-
-/* Note: we always use the highest level.
- * TO boost the performance we may want to add some
- * additional code for level 1
- */
-static int
-gather_random( void (*add)(const void*, size_t, int), int requester,
-					  size_t length, int level )
-{
-    unsigned int result;
-    unsigned int nbytes;
-
-    if( !level )
-	return 0;
-
-    if( !slow_seeder )
-	load_and_init_winseed();
-
-    /* Our estimation on how much entropy we should use is very vague.
-     * Winseed delivers some amount of entropy on each slow poll and
-     * we add it to our random pool.  Depending on the required quality
-     * level we adjust the requested length so that for higher quality
-     * we make sure to add more entropy to our pool.  However, as we don't
-     * like to waste any entropy collected by winseed, we always add
-     * at least everything we got from winseed.
-     */
-    if( level > 1 )
-	length *= 100;
-    else if( level > 0 )
-	length *= 10;
-
-    for(;;) {
-	nbytes = entropy_buffer_size;
-	result = get_seed( slow_seeder, entropy_buffer, &nbytes);
-	if( result == PCP_SEEDER_TOO_SMALL ) {
-	    unsigned int n1 = get_internal_seed_size( slow_seeder );
-
-	    if( n1 > MAX_SEEDER_SIZE ) {
-		g10_log_fatal("rndw32: internal seeder problem (size=%u)\n",
-									  n1);
-		return -1; /* actually never reached */
-	    }
-	    n1 += SEEDER_INC_CHUNK;
-	    set_internal_seed_size( slow_seeder, n1 );
-	    if( n1 > entropy_buffer_size ) {
-		entropy_buffer_size =  n1;
-		entropy_buffer = m_realloc( entropy_buffer,
-					    entropy_buffer_size );
-	    }
-	    continue;
-	}
-
-
-	if( result ) {
-	    g10_log_fatal("rndw32: get_seed(slow) failed: rc=%u\n", result);
-	    return -1; /* actually never reached */
-	}
-	/*g10_log_info("rndw32: slow poll level %d, need %u, got %u\n",
-		      level, (unsigned int)length, (unsigned int)nbytes );*/
-	(*add)( entropy_buffer, nbytes, requester );
-	if( length <= nbytes )
-	    return 0; /* okay */
-	length -= nbytes;
-    }
-}
-
-static int
-gather_random_fast( void (*add)(const void*, size_t, int), int requester )
-{
-    unsigned int result;
-    unsigned int nbytes;
-
-    if( !fast_seeder )
-	load_and_init_winseed();
-
-    /* winseed delivers a constant ammount of entropy for a fast
-     * poll.  We can simply use this and add it to the pool; no need
-     * a loop like it is used in the slow poll */
-    nbytes = entropy_buffer_size;
-    result = get_seed( fast_seeder, entropy_buffer, &nbytes);
-    if( result ) {
-	g10_log_fatal("rndw32: get_seed(fast) failed: rc=%u\n", result);
-	return -1; /* actually never reached */
-    }
-    /*g10_log_info("rndw32: fast poll got %u\n", (unsigned int)nbytes );*/
-    (*add)( entropy_buffer, nbytes, requester );
-    return 0;
-}
-
-#else /* !USE_ENTROPY_DLL */
-/* This is the new code which does not require the entropy.dll */
-
 /*
  * Definitions which are missing from the current GNU Windows32Api
  */
 
+#ifndef TH32CS_SNAPHEAPLIST
 #define TH32CS_SNAPHEAPLIST 1
 #define TH32CS_SNAPPROCESS  2
 #define TH32CS_SNAPTHREAD   4
 #define TH32CS_SNAPMODULE   8
 #define TH32CS_SNAPALL	    (1|2|4|8)
 #define TH32CS_INHERIT	    0x80000000
+#endif /*TH32CS_SNAPHEAPLIST*/
 
+#ifndef IOCTL_DISK_PERFORMANCE
 #define IOCTL_DISK_PERFORMANCE	0x00070020
+#endif
+#ifndef VER_PLATFORM_WIN32_WINDOWS
 #define VER_PLATFORM_WIN32_WINDOWS 1
+#endif
+
+/* This used to be (6*8+5*4+8*2), but Peter Gutmann figured a larger
+   value in a newer release. So we use a far larger value. */
+#define SIZEOF_DISK_PERFORMANCE_STRUCT 256
 
 
 typedef struct {
@@ -556,7 +350,8 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 		log_debug ("rndw32#slow_gatherer_nt: check product options\n" );
 	    status = RegQueryValueEx (hKey, "ProductType", 0, NULL,
 				      szValue, &dwSize);
-	    if (status == ERROR_SUCCESS && stricmp (szValue, "WinNT")) {
+	    if (status == ERROR_SUCCESS
+                && ascii_strcasecmp (szValue, "WinNT")) {
 		/* Note: There are (at least) three cases for ProductType:
 		 * WinNT = NT Workstation, ServerNT = NT Server, LanmanNT =
 		 * NT Server acting as a Domain Controller */
@@ -608,7 +403,7 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 
     /* Get disk I/O statistics for all the hard drives */
     for (nDrive = 0;; nDrive++) {
-	DISK_PERFORMANCE diskPerformance;
+        char diskPerformance[SIZEOF_DISK_PERFORMANCE_STRUCT];
 	char szDevice[50];
 
 	/* Check whether we can access this device */
@@ -621,13 +416,13 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 	/* Note: This only works if you have turned on the disk performance
 	 * counters with 'diskperf -y'.  These counters are off by default */
 	if (DeviceIoControl (hDevice, IOCTL_DISK_PERFORMANCE, NULL, 0,
-			     &diskPerformance, sizeof (DISK_PERFORMANCE),
+			     diskPerformance, SIZEOF_DISK_PERFORMANCE_STRUCT,
 			     &dwSize, NULL))
 	{
 	    if ( debug_me )
 		log_debug ("rndw32#slow_gatherer_nt: iostats drive %d\n",
 								  nDrive );
-	    (*add) ( &diskPerformance, dwSize, requester );
+	    (*add) (diskPerformance, dwSize, requester );
 	}
 	else {
 	    log_info ("NOTE: you should run 'diskperf -y' "
@@ -636,7 +431,7 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 	CloseHandle (hDevice);
     }
 
-  #if 0 /* we don't need this in GnuPG  */
+#if 0 /* we don't need this in GnuPG  */
     /* Wait for any async keyset driver binding to complete.  You may be
      * wondering what this call is doing here... the reason it's necessary is
      * because RegQueryValueEx() will hang indefinitely if the async driver
@@ -656,7 +451,7 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
      * this, we have to wait until any async driver bind has completed
      * before we can call RegQueryValueEx() */
     waitSemaphore (SEMAPHORE_DRIVERBIND);
-  #endif
+#endif
 
     /* Get information from the system performance counters.  This can take
      * a few seconds to do.  In some environments the call to
@@ -684,7 +479,7 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
      * worst-case estimate which is usually nowhere near the actual amount
      * required.  For example it may report that 128K of memory is required,
      * but only return 64K of data */
-    {	pPerfData =  m_alloc (cbPerfData);
+    {	pPerfData =  xmalloc (cbPerfData);
 	for (;;) {
 	    dwSize = cbPerfData;
 	    if ( debug_me )
@@ -701,14 +496,14 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 	    }
 	    else if (status == ERROR_MORE_DATA) {
 		cbPerfData += PERFORMANCE_BUFFER_STEP;
-		pPerfData = m_realloc (pPerfData, cbPerfData);
+		pPerfData = xrealloc (pPerfData, cbPerfData);
 	    }
 	    else {
 		g10_log_debug ( "rndw32: get performance data problem\n");
 		break;
 	    }
 	}
-	m_free (pPerfData);
+	xfree (pPerfData);
     }
     /* Although this isn't documented in the Win32 API docs, it's necessary
        to explicitly close the HKEY_PERFORMANCE_DATA key after use (it's
@@ -719,12 +514,12 @@ slow_gatherer_windowsNT( void (*add)(const void*, size_t, int), int requester )
 }
 
 
-static int
-gather_random( void (*add)(const void*, size_t, int), int requester,
-					  size_t length, int level )
+int
+rndw32_gather_random (void (*add)(const void*, size_t, int), int requester,
+                      size_t length, int level )
 {
     static int is_initialized;
-    static int is_windows95;
+    static int is_windowsNT, has_toolhelp;
 
 
     if( !level )
@@ -742,7 +537,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 
 	GetVersionEx( &osvi );
 	platform = osvi.dwPlatformId;
-	is_windows95 = platform == VER_PLATFORM_WIN32_WINDOWS;
+        is_windowsNT = platform == VER_PLATFORM_WIN32_NT;
+        has_toolhelp = (platform == VER_PLATFORM_WIN32_WINDOWS
+                        || (is_windowsNT && osvi.dwMajorVersion >= 5));
 
 	if ( platform == VER_PLATFORM_WIN32s ) {
 	    g10_log_fatal("can't run on a W32s platform\n" );
@@ -757,11 +554,11 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	log_debug ("rndw32#gather_random: req=%d len=%u lvl=%d\n",
 			   requester, (unsigned int)length, level );
 
-    if (is_windows95 ) {
-	slow_gatherer_windows95( add, requester );
+    if ( has_toolhelp ) {
+        slow_gatherer_windows95 ( add, requester );
     }
-    else {
-	slow_gatherer_windowsNT( add, requester );
+    if ( is_windowsNT ) {
+        slow_gatherer_windowsNT ( add, requester );
     }
 
     return 0;
@@ -769,8 +566,8 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 
 
 
-static int
-gather_random_fast( void (*add)(const void*, size_t, int), int requester )
+int
+rndw32_gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 {
     static int addedFixedItems = 0;
 
@@ -788,7 +585,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
      * events in input queue, and milliseconds since Windows was started */
     {	byte buffer[20*sizeof(ulong)], *bufptr;
 	bufptr = buffer;
-      #define ADD(f)  do { ulong along = (ulong)(f);		      \
+#define ADD(f)  do { ulong along = (ulong)(f);		      \
 			   memcpy (bufptr, &along, sizeof (along) );  \
 			   bufptr += sizeof (along); } while (0)
 	ADD ( GetActiveWindow ());
@@ -812,7 +609,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 
 	assert ( bufptr-buffer < sizeof (buffer) );
 	(*add) ( buffer, bufptr-buffer, requester );
-      #undef ADD
+#undef ADD
     }
 
     /* Get multiword system information: Current caret position, current
@@ -855,13 +652,14 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 	(*add) ( &kernelTime, sizeof (kernelTime), requester );
 	(*add) ( &userTime, sizeof (userTime), requester );
 
-	/* Get the minimum and maximum working set size for the current process */
+	/* Get the minimum and maximum working set size for the
+           current process */
 	GetProcessWorkingSetSize (handle, &minimumWorkingSetSize,
 					  &maximumWorkingSetSize);
 	(*add) ( &minimumWorkingSetSize,
-				   sizeof (&minimumWorkingSetSize), requester );
+				   sizeof (minimumWorkingSetSize), requester );
 	(*add) ( &maximumWorkingSetSize,
-				   sizeof (&maximumWorkingSetSize), requester );
+				   sizeof (maximumWorkingSetSize), requester );
     }
 
 
@@ -889,7 +687,7 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 	if (QueryPerformanceCounter (&performanceCount)) {
 	    if ( debug_me )
 		log_debug ("rndw32#gather_random_fast: perf data\n");
-	    (*add) (&performanceCount, sizeof (&performanceCount), requester);
+	    (*add) (&performanceCount, sizeof (performanceCount), requester);
 	}
 	else { /* Millisecond accuracy at best... */
 	    DWORD aword = GetTickCount ();
@@ -899,60 +697,3 @@ gather_random_fast( void (*add)(const void*, size_t, int), int requester )
 
     return 0;
 }
-
-
-
-
-
-#endif /* !USE_ENTROPY_DLL */
-
-
-#ifndef IS_MODULE
-static
-#endif
-const char * const gnupgext_version = "RNDW32 ($Revision: 1.4 $)";
-
-static struct {
-    int class;
-    int version;
-    void *func;
-} func_table[] = {
-    { 40, 1, gather_random },
-    { 41, 1, gather_random_fast },
-};
-
-
-#ifndef IS_MODULE
-static
-#endif
-void *
-gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
-{
-    void *ret;
-    int i = *sequence;
-
-    debug_me = !!getenv("DEBUG_RNDW32");
-
-    do {
-	if ( i >= DIM(func_table) || i < 0 ) {
-	    return NULL;
-	}
-	*class = func_table[i].class;
-	*vers  = func_table[i].version;
-	ret = func_table[i].func;
-	i++;
-    } while ( what && what != *class );
-
-    *sequence = i;
-    return ret;
-}
-
-#ifndef IS_MODULE
-void
-rndw32_constructor(void)
-{
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
-}
-#endif
-
